define([
    // dojo
    "dojo/_base/declare",
    "dojo/aspect",
    "dojo/when",
    "dojo/Deferred",

    // epi.cms
    "epi-cms/core/PermanentLinkHelper",
    "epi-cms/contentediting/editors/LinkItemEditor",
    "./SelectorWrapperBase",

    // resources
    "epi/i18n!epi/cms/nls/episerver.cms.widget.editlink"
], function (
    // dojo
    declare,
    aspect,
    when,
    Deferred,

    // epi.cms
    PermanentLinkHelper,
    LinkItemEditor,
    SelectorWrapperBase,

    // resources
    res
) {

    return declare([SelectorWrapperBase], {
        // summary:
        //    Represents the widget wrapper.
        // tags:
        //      internal xproduct

        resource: res,

        selectorClass: LinkItemEditor,

        hiddenFields: ["text", "title", "target", "language"],

        // LinkHelper to retrieve the link info
        linkHelper: PermanentLinkHelper,

        _getSelectorParams: function () {
            // summary:
            //      Retrieves the selector's params.
            //
            // tags:
            //      protected abstract

            // should implement by usage or inheritance class.
            return {
                hiddenFields: this.hiddenFields,
                getCurrentValue: function () {
                    return { href: this.value };
                }.bind(this)
            };
        },

        _onSelectorValueChange: function (val) {
            // summary:
            //      Callback whenever selector's value is changed.
            //
            // tags:
            //      protected abstract

            if ((!val && this.value) || (val && this.value !== val.href)) {
                this.onFocus();
                this.set("value", val && val.href);
                this._triggerSave();
            }
        },

        _setValue: function (value) {
            this.value = value;
            when(this._getLinkData(value)).then(function (linkData) {
                this.selector && this.selector.set("value", linkData);
            }.bind(this));
        },

        _getLinkData: function (/* String */ value) {
            // summary:
            //      Retrieves the content of given link value.
            //
            // value: [String]
            //    A string as link value
            //
            // tags:
            //      private

            if (!value) {
                return null;
            }

            var def = new Deferred();
            when(this.linkHelper.getContent(value, { allLanguages: true }), function (content) {
                var result = content ? {
                    text: content.name,
                    href: content.permanentLink
                } : {
                    text: value,
                    href: value
                };
                if (!def.isCanceled()) {
                    def.resolve(result);
                }
            }.bind(this));

            return def.promise;
        }
    });
});
